/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      WDM CSA+ Network source file 
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |	Include definitions bellow this line
 +--------------------------------------------------------------------*/
#include "wdm_csa_p_network.h"
#include "wdm_csa_p_station.h"
#include "assert.h"
#include "malloc.h"
#include "stdlib.h"


/*--------------------------------------------------------------------
 |	Function bodies bellow this line
 +--------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 |	CWDM_CSA_P_Network :  public constructor
 +--------------------------------------------------------------------*/
CWDM_CSA_P_Network::CWDM_CSA_P_Network( long N_station, long k_network, double  p_arr ) : 
    CWDM_Network( N_station, k_network, p_arr )
{
    int i = 0;

    /* creates the N CSA_stations of the network */
    WS  = ( CWDM_Station ** )malloc( N_station * sizeof( CWDM_CSA_P_Station * ) );
    assert( WS );
    for ( i = 0; i < N_station; i++ ) {
        WS[i]   = new CWDM_CSA_P_Station( i, N_station, k_network, p_arr );
        assert( WS );
    }
}



/*--------------------------------------------------------------------
 |	~CWDM_CSA_P_Network : public virtual destructor
 +--------------------------------------------------------------------*/
CWDM_CSA_P_Network::~CWDM_CSA_P_Network( void )
{
    int i = 0;
    
    /* deletes the N CSA station created above */
    for ( i = 0; i < N; i++ ) {
        delete WS[i];
    }
    free( WS );
    // C++ calls base class default destructor automatically after 
}


/*----------------------------------------------------------------------------------
 |	simulate :  simulates the activity of the network for a simu_length slot period
 +----------------------------------------------------------------------------------*/
void CWDM_CSA_P_Network::simulate( long simu_length )
{
    int     i = 0;
    long    old_date    = this->date;
    long    r_pos       = 0;    // receiver pos
    long    t_pos       = 0;    // transmission pos
    long    ct_pos      = 0;    // current transmitter pos
    long    rq_pos      = 0;    // reception queue pos
    WDM_Slot    dummy = { false, 0, 0, 0 };

    for ( ; this->date < old_date + simu_length; this->date ++ ) {
        /* generates the packets  */
        for ( i = 0; i < N; i++ ) {
            WS[i]->generate_packet( this->date, SP.npg, SP.npb, SP.npl );
        }
        // Caution : Simulation Probe
        for ( i = 0; i < N*N; i ++ ) {
            SP.nsr  += ( CS[0][i] == 1 ) ? 1 : 0;
        }
        /* runs the CSA algorithm */
        { // CSA algorithm 
            ct_pos = t_pos;
            do {
                /* chooses a valid transmission queue of WS[ct_pos] starting at r_pos */
                rq_pos = r_pos;
                while ( ( r_pos && rq_pos != r_pos - 1 || !r_pos && rq_pos != N - 1) && 
                        CS[0][ ct_pos * N + rq_pos ] != 1 ) {
                    rq_pos  = ( rq_pos + 1 ) % N;
                }
                if ( CS[0][ ct_pos * N + rq_pos ] == 1) {
                    /* sends the first packet of TB[rq_pos] of WS[ct_pos] */
                    DS[ct_pos]  = WS[ct_pos]->send_packet( rq_pos, 0 );
                    assert( DS[ct_pos].occupied );
                    // Caution : Simulation Probe
                    SP.npt++;
                    SP.nra++;
                    SP.tsd += ( this->date - DS[ct_pos].arrival_date );
                }
                else {
                    // Caution : Simulation Probe
                    SP.nes++;
                }
                /* invalidates virtually TB[rq_pos] in all WS */
                for ( i = 0; i < N; i++ ) {
                    CS[0][ i * N + rq_pos ] = -1;
                }
                /* changes ct_pos to the next transmitter */
                ct_pos = ( ct_pos + 1 ) % N;
            } while ( ct_pos != t_pos );
            /* changes t_pos to the next transmitter */
            t_pos   = ( t_pos + 1 ) % N;
            /* changes r_pos to the next receiver */
            r_pos   = ( t_pos ) ? r_pos : ( r_pos + 1 ) % N;
        }
		/* generates a new control slot */
        for ( i = 0; i < N; i++ ) {        
            WS[i]->update_control_slot( CS[k] );
        }
        /* propagates the control slot for one slot*/
        this->propagate_control_slot();
        /* updates the simulation probe */
        SP.nsg  += N;
        SP.q    = ( SP.nsr ) ?  1.0 * SP.nra / SP.nsr   : SP.q;
        SP.ro   = ( SP.nsg ) ?  1.0 * SP.npg / SP.nsg   : SP.ro;
        SP.asd  = ( SP.npt ) ?  1.0 * SP.tsd / SP.npt   : SP.asd;
        SP.apl  = ( SP.npg ) ?  1.0 * SP.npl / SP.npg   : SP.apl;
        SP.ant  = ( SP.nsg ) ?  1.0 * SP.npt / SP.nsg   : SP.ant;
    }
}
