/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      WDM CSA+ Station source file 
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |	Include definitions bellow this line
 +--------------------------------------------------------------------*/
#include "wdm_csa_p_station.h"
#include "assert.h"
#include "stdlib.h"

/*--------------------------------------------------------------------
 |	Function bodies bellow this line
 +--------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 |	CWDM_CSA_P_Station : public constructor
 +--------------------------------------------------------------------*/
CWDM_CSA_P_Station::CWDM_CSA_P_Station( long station_id, long N_station, long k_net, double p_arr ) :
    CWDM_Station( station_id, N_station, k_net, p_arr )
{
	int i = 0;
	// inits the TBC
	TBC	= ( char ** )malloc(  N_station * sizeof( char * ) );
	assert(TBC);
	for ( i = 0; i < N; i++ ) {
		TBC[i]	= ( char * )malloc( k_net * sizeof( char ) );
		assert( TBC[i] );
	}
}


 /*--------------------------------------------------------------------
 |	~CWDM_CSA_P_Station : public virtual destructor
 +--------------------------------------------------------------------*/
CWDM_CSA_P_Station::~CWDM_CSA_P_Station( void )
{
	int i = 0;

	for ( i = 0; i < k; i++ ) {
		free( TBC[i] );
	}
	free( TBC );
    // C++ calls default base class destructor automatically after
}



/*--------------------------------------------------------------------
 |	reset : resets the station and its transmission buffer
 +--------------------------------------------------------------------*/
void CWDM_CSA_P_Station::reset( double p_arr )
{
    int         i = 0;  
    int         j = 0;  
    WDM_Slot    dummy = { false, 0, 0, 0 };

    this->p = p_arr;
	// resets the TB
    for ( i = 0; i < N; i++ ) {
        for ( j = 0; j < N*(k+1); j++ ) {
            TB[i][j]  = dummy;
        }
    }
	// resets the TBC
	for ( i = 0; i < N; i++ ) {
		for ( j = 0; j < k; j++ ) {
			TBC[i][j] = 0;
		}
	}	
}



/*-----------------------------------------------------------------------------
 |	upadte_control_slot :   updates the control minislot of the sation   
 +-----------------------------------------------------------------------------*/
void CWDM_CSA_P_Station::update_control_slot( char *control_slot )
{
    int i   = 0;
	int j	= 0;
    int msb = this->id * N; // minislot base
	int	sig_pos = 0;

    assert( control_slot );
	// shifts the TBCs
    for ( i = 0; i < N; i++ ) {
		for ( j = 1; j < k; j ++) {
			TBC[i][j-1] = TBC[i][j];
		}
		TBC[i][k-1]	= 0;
	}
	// post a new res if needed for each TB
	for ( i = 0; i < N; i++ ) {
		int nso = 0;	// number of slot occupied
		int	nrp = 0;	// number of reservation pending
		for ( j = 0; j < k; j ++) {
			nso += TB[i][j].occupied ? 1 : 0;
			nrp += TBC[i][j] ? 1 : 0;
		}
		TBC[i][k-1] = ( nso > nrp ) ? 1 : 0;
	}
	// updates the control slot
	for( i = 0; i < N; i++ ) {
		control_slot[ msb + i ] = TBC[i][k-1];
    }
}
