/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      WDM Network source file 
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |	Include definitions bellow this line
 +--------------------------------------------------------------------*/
#include "wdm_network.h"
#include "assert.h"
#include "malloc.h"
#include "stdlib.h"
#include "time.h"

/*--------------------------------------------------------------------
 |	Function bodies bellow this line
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |	CWDM_Network : public constructor
 +--------------------------------------------------------------------*/
CWDM_Network::CWDM_Network( long N_station, long k_network, double  p_arr ) :
    N( N_station ), k( k_network ), p( p_arr )
{
    int i = 0;

    /* checks the basic parameters */
    assert( N_station > 0 );
    assert( k_network > 0 );
    /* allocation of the k + 1 control slots */
    CS  = ( char ** )malloc( ( k_network + 1 ) * sizeof( char *) );
    assert( CS );
    for ( i = 0; i < k + 1; i++ )  {
        CS[i] = ( char * )malloc( N_station * N_station * sizeof( char ) );
        assert( CS[i] );
    }
    /* allocation of the N data slots */
    DS  = ( WDM_Slot * ) malloc( N_station * sizeof( WDM_Slot ) );
    assert( DS );
}


/*--------------------------------------------------------------------
 |	~CWDM_Network : public virtual destructor
 |                  must be redifined in derived classes 
 +--------------------------------------------------------------------*/
CWDM_Network::~CWDM_Network( void )
{
    int i = 0;

    /* deletes the k+1 control slots */
    for ( i = 0; i < k + 1; i ++ ) {
        free( CS[i] );
    }
    free( CS );
    /* deallocates the N data slots */
    free( DS );
}


/*--------------------------------------------------------------------
 |	reset : resets the network and all the simulation variables needed
 +--------------------------------------------------------------------*/
void CWDM_Network::reset( double p_arr )
{
    int i = 0;
    int j = 0;

    this->p     = p_arr;
    this->date  = 0;
    /* resseds the random number generator */
    srand( (unsigned)time( NULL ) );
    /* resets all the WS */
    for ( i = 0; i < N; i++ ) {
        WS[i]->reset( p_arr );
    }
    /* resets all the control slots */
    for ( i = 0; i < k + 1; i ++) {
        for ( j = 0; j < N*N; j ++ ) {
            CS[i][j] = 0;
        }
    }
    /* resets the probe */
    SP.nes  = 0;
    SP.nsg  = 0;
    SP.npg  = 0;
    SP.npb  = 0;
    SP.npl  = 0;
    SP.npt  = 0;
    SP.nsr  = 0;
    SP.nra  = 0;
    SP.tsd  = 0;
    SP.q    = 0.0;
    SP.ro   = 0.0;
    SP.asd  = 0.0;
    SP.apl  = 0.0;
    SP.ant  = 0.0;
}
    


/*--------------------------------------------------------------------
 |	propagate_control_slot :    propagates the control slots for one slot
 +--------------------------------------------------------------------*/
void CWDM_Network::propagate_control_slot( void )
{
    int i = 0;
    int j = 0;

    /* shifts up the control slots */
    for ( i = 1; i < k + 1; i++ ) {
        for ( j = 0; j < N*N; j++ ) {
            CS[i-1][j]  = CS[i][j];
        }
    }
    /* resets the bottom line */
    for ( j = 0; j < N*N; j++ ) {
        CS[k][j] = 0;
    }
}




/*----------------------------------------------------------------------------------
 |	simulate :  simulates the activity of the network for a simu_length slot period
 |              must be redifined in derived classes
 +----------------------------------------------------------------------------------*/
void CWDM_Network::simulate( long simu_length )
{
    /* default implementation does nothing */
}
