/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      WDM Network header file 
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/

#ifndef _WDM_NETWORK_H_
#define _WDM_NETWORK_H_


/*--------------------------------------------------------------------
 |	Include definitions bellow this line
 +--------------------------------------------------------------------*/
#include "wdm_station.h"

/*--------------------------------------------------------------------
 |  Type definitions bellow this line
 +--------------------------------------------------------------------*/
typedef struct {
    long            nes;    /* total number of of empty slot generated */
    long            nsg;    /* total number of slot generated */            
    long            npg;    /* total number of packet generated */
    long            npb;    /* total number of packet buffered */
    long            npl;    /* total number of packet lost */
    long            npt;    /* total number of packet transmitted */
    long            nsr;    /* total number of slots reserved */
    long            nra;    /* total number of reservation accepted */
    long            tsd;    /* total slot delay */
    double          q;      /* probability of a packet to be accepted by the MAC algorithm */    
    double          ro;     /* offered load */
    double          asd;    /* average slot delay */
    double          apl;    /* average packet loss */    
    double          ant;    /* average network throughput*/
} WDM_Probe;


/*--------------------------------------------------------------------
 |  Class definitions bellow this line
 +--------------------------------------------------------------------*/

class CWDM_Network 
{

protected :
    bool            pipelined;  /* indicates if the network is in pipelined mode */
    long            date;       /* network simulation time */
    WDM_Probe       SP;         /* the probe used to follow the simulation */
    char            **CS;       /* array of k+1 control slots */
    WDM_Slot        *DS;        /* array of N data slots */
    CWDM_Station    **WS;       /* array of N WDM_Station */
    
public :
    long    N;  /* number of stations int he network */
    long    k;  /* k = E[ Roundrip delay / slot duration + 1 ] */
    double  p;  /* arrival probability of a paquet in a transmission buffer */

    /* public constructor */
    CWDM_Network( long N_station, long k_network, double  p_arr );

    /* public virtual destructor */
    /*  must be redifined in derived classes */
    virtual ~CWDM_Network( void );

    /*  resets the network and all the simulation variables needed */
    void reset( double p_arr );

    /*  propagates the control slots for one slot */
    void propagate_control_slot( void );

    /* simulates the activity of the network for a simu_length slot period */
    /*  must be redifined in derived classes */
    virtual void simulate( long simu_length );

    /* retrieves a pointer to the probe structure used in the simulation process */
    inline WDM_Probe *get_probe( void ) { return &SP; };

};


#endif  // _WDM_NETWORK_H_