/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      WDM Station source file 
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |	Include definitions bellow this line
 +--------------------------------------------------------------------*/
#include "wdm_station.h"
#include "assert.h"
#include "malloc.h"
#include "stdlib.h"
#include "stdio.h"

/*--------------------------------------------------------------------
 |	Function bodies bellow this line
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |	CWDM_Station : public constructor
 +--------------------------------------------------------------------*/
CWDM_Station::CWDM_Station( long station_id, long N_station, long k_net, double p_arr ) : 
    id( station_id ), N( N_station ), k( k_net ), p( p_arr )
{
    int i = 0;

    TB  = (WDM_Slot **)malloc( N_station * sizeof( WDM_Slot * ) );
    assert( TB );
    for ( i = 0; i < N_station; i++ ) {
        TB[i]   = (WDM_Slot *)malloc( N_station * ( k_net + 1 ) * sizeof( WDM_Slot ) );
        assert( TB[i] );
    }
}


/*--------------------------------------------------------------------
 |	~CWDM_Station : public virtual destructor
 |                  must be redifined in derived classes 
 +--------------------------------------------------------------------*/
CWDM_Station::~CWDM_Station( void )
{
    int i = 0;

    for ( i = 0; i < N; i ++ ) {
        free( TB[i] );
    }
    free( TB );
}


/*--------------------------------------------------------------------
 |	reset : resets the station and its transmission buffer
 +--------------------------------------------------------------------*/
void CWDM_Station::reset( double p_arr )
{
    int         i = 0;  
    int         j = 0;  
    WDM_Slot    dummy = { false, 0, 0, 0 };

    this->p = p_arr;
    for ( i = 0; i < N; i++ ) {
        for ( j = 0; j < N*(k+1); j++ ) {
            TB[i][j]  = dummy;
        }
    }
}


/*-----------------------------------------------------------------------------
 |	generate_packet :   generates packets in the TBs : must be called at each 
 |                      slot 
 +-----------------------------------------------------------------------------*/
void CWDM_Station::generate_packet( long simu_date, long &npg, long &npb, long &npl  )
{
    int         i = 0;
    int         j = 0;
    WDM_Slot    packet_slot = { true , 0, 0, simu_date };

    for ( i = 0; i < N; i++ ) {
        if ( ( 1.0 * rand() / RAND_MAX ) < p ) {
            npg++;
            /* we can generate a packet in transmission file i */
            j = 0;
            /* locates the first available slot in TB[i] */
            while ( j < N*(k+1) && TB[i][j].occupied ) j++;
            /* if an empty slot was found  creates a packet */
            if ( j < N*(k+1) ) {
                npb++;
                packet_slot.trans   = this->id;
                packet_slot.recv    = i;
                TB[i][j]    = packet_slot;
            }
            else {
                //printf( " WS[%d].TB[%d] overflow.\n ", this->id, i );
                npl++;
            }
        }
    }
}


/*-----------------------------------------------------------------------------
 |	update_control_slot :   updates the control minislot of the sation   
 |                          must be redifined in derived classes 
 +-----------------------------------------------------------------------------*/
void CWDM_Station::update_control_slot( char *control_slot )
{
    assert( control_slot );
    /* do nothing */
}


/*-----------------------------------------------------------------------------
 |	send_packet :   sends the packet selected by the scheduling algorithm 
 +-----------------------------------------------------------------------------*/
WDM_Slot CWDM_Station::send_packet( long TB_pos, long packet_pos )
{
    int i = 0;
    WDM_Slot    dummy_packet    = { false, 0, 0, 0 };
    WDM_Slot    data_packet;

    if (    TB_pos  < 0 || TB_pos >= N || 
            packet_pos < 0 || packet_pos >= N*(k+1) ) return dummy_packet;
    /* retrieves the selected packet from the TB */
    data_packet = TB[TB_pos][packet_pos];
    /* shifts all the packets of TB[TB_pos] to the left starting from packet_pos*/
    for ( i = packet_pos + 1; i < N*(k+1); i++ ) {
         TB[TB_pos][i-1] = TB[TB_pos][i];
    }
    TB[TB_pos][N*(k+1)-1]   = dummy_packet;
    return  data_packet;
}


