/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      WDM Station header file 
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/



#ifndef _WDM_STATION_H_
#define _WDM_STATION_H_


/*--------------------------------------------------------------------
 |	Include definitions bellow this line
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |  Type definitions bellow this line
 +--------------------------------------------------------------------*/

    /*** WDM Slot definition ***/
typedef struct {
    bool    occupied;       /* indicates if the transmission slot is occupied */
    long    trans;          /* transmitter's id */
    long    recv;           /* receiver's id */
    long    arrival_date;   /* arrival date of the packet in this transmission slot*/
} WDM_Slot;


/*--------------------------------------------------------------------
 |  Class definitions bellow this line
 +--------------------------------------------------------------------*/

    /*** WDM Station class definition ***/
class CWDM_Station 
{

protected :
    WDM_Slot    **TB;    /* transmision buffers */

public :
    long    N;          /* number of station in the network */
    long    id;         /* id of thos sation */
    long    k;          /* network spawn in slot */
    double  p;          /* arrival probability of a paquet in a transmission buffer */

    /*  public constructor */
    CWDM_Station( long station_id, long N_station, long k_net, double p_arr );

    /* public virtual destructor */
    /*  must be redifined in derived classes */
    virtual ~CWDM_Station( void );

    /*  resets the station and its transmission buffer */
    void reset( double p_arr );

    /*  generates packets in the TBs : must be called at each slot */
    /*  returns the number of packets generated */
    void generate_packet( long simu_date, long &npg, long &npb, long &npl  );

    /*  updates the control minislot of the sation */
    /*  must be redifined in derived classes */
    virtual void update_control_slot( char *control_slot );

    /*  runs the scheduling algorithm and sends a packet    */
    WDM_Slot send_packet( long TB_pos, long packet_pos );

};  /* end class WDM_Station */


#endif // _WDM_STATION_H_
