/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      WDM test source file 
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/


/*--------------------------------------------------------------------
 |	Include definitions bellow this line
 +--------------------------------------------------------------------*/
#include "wdm_csa_p_network.h"
#include "wdm_csa_network.h"
#include "wdm_rsa_network.h"
#include <stdio.h>
#include <assert.h>


/*--------------------------------------------------------------------
 |	Function bodies bellow this line
 +--------------------------------------------------------------------*/



void print_res( WDM_Probe   *probe )
{
	assert( probe );
	printf("-> Simulation done, printing results...\n\n");
	printf("\tnsg = %9ld    nes = %9ld\n", probe->nsg, probe->nes);    
	printf("\tnpg = %9ld    npt = %9ld\n", probe->npg, probe->npt);    
	printf("\tnpb = %9ld    npl = %9ld\n", probe->npb, probe->npl);    
	printf("\tnsr = %9ld    nra = %9ld\n", probe->nsr, probe->nra);    
	printf("\ttsd = %9ld    q   = %9f\n", probe->tsd, probe->q);    
	printf("\tro  = %9f    asd = %9f\n", probe->ro, probe->asd);    
	printf("\tant = %9f    apl = %9f\n", probe->ant, probe->apl);    
}


void main( void )
{
	char		ans;
    long        N   = 0;
    long        k   = 0;
    long        sl  = 0;
    float       p   = 0.0;
    WDM_Probe   *probe = NULL;    
    CWDM_RSA_Network    *rsa_net	= NULL;
    CWDM_CSA_Network    *csa_net	= NULL;
    CWDM_CSA_P_Network  *csa_p_net	= NULL;
    
    printf("\n**** WDM Test program ****\n\n");
    /* reading network parameters */
    printf("Number of Station N = ");
    scanf("%d", &N );
    printf("Network size ( in slots ) k = ");
    scanf("%d", &k );
    printf("Transmission queue arrival probability per slot = ");
    scanf("%f", &p );
    /* reading simulation parameter */
    printf("Simulation length sl = ");
    scanf("%d", &sl );
	/* simulates a RSA network */
	printf("\nSimulate RSA Network ( y / n ) ?");
	fflush( stdin );
    scanf("%c", &ans );
    if ( ans == 'y' || ans == 'Y' ) {
		/* creating network */
		printf("\n-> Creating RSA Network N = %ld, k = %ld, p = %f ...\n", N, k, p);
		rsa_net = new CWDM_RSA_Network( N, k, (double)p );
		/* simulating network */
		printf("-> Simulating RSA Network for %ld slots duration...\n", sl);
		rsa_net->reset( (double)p );
		rsa_net->simulate( sl );
		/* printing simulation result */
		probe   = rsa_net->get_probe();
		print_res( probe );
		/* deleting network */
		printf("\n-> deleting RSA Network...\n");
		delete rsa_net;
	}
	/* simulates a CSA network */
	printf("\nSimulate CSA Network ( y / n ) ?");
	fflush( stdin );
    scanf("%c", &ans );
    if ( ans == 'y' || ans == 'Y' ) {
		/* creating network */
		printf("\n-> Creating CSA Network N = %ld, k = %ld, p = %f ...\n", N, k, p);
		csa_net = new CWDM_CSA_Network( N, k, (double)p );
		/* simulating network */
		printf("-> Simulating CSA Network for %ld slots duration...\n", sl);
		csa_net->reset( (double)p );
		csa_net->simulate( sl );
		/* printing simulation result */
		probe   = csa_net->get_probe();
		print_res( probe );
		/* deleting network */
		printf("\n-> deleting CSA Network...\n");
		delete csa_net;
	}
	/* simulates a CSA+ network */
	printf("\nSimulate CSA+ Network ( y / n ) ?");
	fflush( stdin );
    scanf("%c", &ans );
    if ( ans == 'y' || ans == 'Y' ) {
		/* creating network */
		printf("\n-> Creating CSA+ Network N = %ld, k = %ld, p = %f ...\n", N, k, p);
		csa_p_net = new CWDM_CSA_P_Network( N, k, (double)p );
		/* simulating network */
		printf("-> Simulating CSA+ Network for %ld slots duration...\n", sl);
		csa_p_net->reset( (double)p );
		csa_p_net->simulate( sl );
		/* printing simulation result */
		probe   = csa_p_net->get_probe();
		print_res( probe );
		/* deleting network */
		printf("\n-> deleting CSA+ Network...\n");
		delete csa_p_net;
	}
    printf("\nbye.\n");
}




