/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      Simu_monitor_dlg.cpp : implementation file
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/



// Simu_monitor_dlg.cpp : implementation file
//

#include "stdafx.h"
#include "wsasim.h"
#include "Simu_monitor_dlg.h"
#include "assert.h"
#include "..\sasim\wdm_rsa_network.h"
#include "..\sasim\wdm_csa_network.h"
#include "..\sasim\wdm_csa_p_network.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSimu_monitor_dlg dialog


#define MONITOR_TIMER_ID    32
#define MONITOR_TIMER_DELAY 500


CSimu_monitor_dlg::CSimu_monitor_dlg(CWnd* pParent /*=NULL*/, WDM_Simu_Context *SC )
	: CDialog(CSimu_monitor_dlg::IDD, pParent)
{
    assert( SC );
    this->SC = SC;
	//{{AFX_DATA_INIT(CSimu_monitor_dlg)
	m_algo = _T("");
	m_ant = 0.0;
	m_apl = 0.0;
	m_asd = 0.0;
	m_n_size = SC->net_param.n_size;
	m_n_station = SC->net_param.n_station;
	m_nes = 0;
	m_npb = 0;
	m_npg = 0;
	m_npl = 0;
	m_npt = 0;
	m_nra = 0;
	m_nsg = 0;
	m_nsr = 0;
	m_point = 0;
	m_q = 0.0;
	m_ro = 0.0;
	m_tsd = 0;
	//}}AFX_DATA_INIT
}


void CSimu_monitor_dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSimu_monitor_dlg)
	DDX_Control(pDX, IDC_simu_progress, m_simu_progress);
	DDX_Text(pDX, IDC_Algo_mon, m_algo);
	DDX_Text(pDX, IDC_ant_mon, m_ant);
	DDX_Text(pDX, IDC_apl_mon, m_apl);
	DDX_Text(pDX, IDC_asd_mon, m_asd);
	DDX_Text(pDX, IDC_N_Size_mon, m_n_size);
	DDX_Text(pDX, IDC_N_Station_mon, m_n_station);
	DDX_Text(pDX, IDC_nes_mon, m_nes);
	DDX_Text(pDX, IDC_npb_mon, m_npb);
	DDX_Text(pDX, IDC_npg_mon, m_npg);
	DDX_Text(pDX, IDC_npl_mon, m_npl);
	DDX_Text(pDX, IDC_npt_mon, m_npt);
	DDX_Text(pDX, IDC_nra_mon, m_nra);
	DDX_Text(pDX, IDC_nsg_mon, m_nsg);
	DDX_Text(pDX, IDC_nsr_mon, m_nsr);
	DDX_Text(pDX, IDC_Point_mon, m_point);
	DDX_Text(pDX, IDC_q_mon, m_q);
	DDX_Text(pDX, IDC_ro_mon, m_ro);
	DDX_Text(pDX, IDC_tsd_mon, m_tsd);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSimu_monitor_dlg, CDialog)
	//{{AFX_MSG_MAP(CSimu_monitor_dlg)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSimu_monitor_dlg message handlers


// the true simulation function
DWORD WINAPI simulate( LPVOID param ) 
{
    UINT    i   = 0;
    WDM_Simu_Context *SC_global     = (WDM_Simu_Context *)param;
    assert(SC_global);
    SC_global->simu_state.running   = TRUE;
    // add simulation code here
	//
    /* sets up the default parameters */
    SC_global->simu_mon.c_algo  = 0;
    SC_global->simu_mon.n_algo  = 0;
    if ( SC_global->simu_param.rsa )    SC_global->simu_mon.n_algo++;
    if ( SC_global->simu_param.csa )    SC_global->simu_mon.n_algo++;
    if ( SC_global->simu_param.csa_p )  SC_global->simu_mon.n_algo++;
    SC_global->simu_mon.c_probe     = NULL;
    SC_global->simu_mon.c_point     = 0;
    SC_global->simu_mon.c_algo_name = " ";
    UINT    N   = SC_global->net_param.n_station;
    UINT    k   = SC_global->net_param.n_size;
    UINT    np  = SC_global->simu_param.npoint;
    double  p   = 0.0;
    WDM_Probe *res  = 
        ( WDM_Probe * )malloc( SC_global->simu_mon.n_algo * np * sizeof( WDM_Probe ) );
    assert( res );
    SC_global->simu_mon.pp_probe    = ( WDM_Probe * )malloc( sizeof( WDM_Probe ) );
    assert( SC_global->simu_mon.pp_probe );
	int ai  = 0;    // algo_index
    /* simulates an RSA network if needed */
    if ( SC_global->simu_param.rsa ) {
        SC_global->simu_mon.c_algo++;
        SC_global->simu_mon.c_algo_name = "RSA";
        for ( i = 0; i < np; i++ ) {
            if ( SC_global->simu_state.canceled ) goto Simu_canceled;
            p   = 1.0 * ( i + 1 ) / (  np * N );
            SC_global->simu_mon.c_point     = i;
            // change this line in the final version !!!!
            CWDM_RSA_Network    *rsa_net    = new CWDM_RSA_Network( N, k, p );
            rsa_net->reset( p );
            SC_global->simu_mon.c_probe = rsa_net->get_probe();
            assert( SC_global->simu_mon.c_probe != NULL );
            rsa_net->simulate( SC_global->simu_param.nslot / N );
            res[ ai*np + i]  = (*(SC_global->simu_mon.c_probe));
            (*(SC_global->simu_mon.pp_probe))   = (*(SC_global->simu_mon.c_probe));
            SC_global->simu_mon.c_probe         = NULL;
            delete rsa_net;
        }
		ai++;
    }
    /* simulates a CSA network if needed */
    if ( SC_global->simu_param.csa ) {
        SC_global->simu_mon.c_algo++;
        SC_global->simu_mon.c_algo_name = "CSA";
        for ( i = 0; i < np; i++ ) {
            if ( SC_global->simu_state.canceled ) goto Simu_canceled;
            p   = 1.0 * ( i + 1 ) / (  np * N );
            SC_global->simu_mon.c_point     = i;
            CWDM_CSA_Network    *csa_net    = new CWDM_CSA_Network( N, k, p );
            csa_net->reset( p );
            SC_global->simu_mon.c_probe = csa_net->get_probe();
            assert( SC_global->simu_mon.c_probe != NULL );
            csa_net->simulate( SC_global->simu_param.nslot / N );
            res[ ai*np + i ]  = (*(SC_global->simu_mon.c_probe));
            (*(SC_global->simu_mon.pp_probe))   = (*(SC_global->simu_mon.c_probe));
            SC_global->simu_mon.c_probe         = NULL;
            delete csa_net;
        }
		ai++;
    }
    /* simulates a CSA+ network if needed */
    if ( SC_global->simu_param.csa_p ) {
        SC_global->simu_mon.c_algo++;
        SC_global->simu_mon.c_algo_name = "CSA+";
        for ( i = 0; i < np; i++ ) {
            if ( SC_global->simu_state.canceled ) goto Simu_canceled;
            p   = 1.0 * ( i + 1 ) / (  np * N );
            SC_global->simu_mon.c_point     = i;
            // change this line in the final version !!!!
            CWDM_CSA_P_Network    *csa_p_net  = new CWDM_CSA_P_Network( N, k, p );
            csa_p_net->reset( p );
            SC_global->simu_mon.c_probe = csa_p_net->get_probe();
            assert( SC_global->simu_mon.c_probe != NULL );
            csa_p_net->simulate( SC_global->simu_param.nslot / N );
            res[ ai*np + i ]  = (*(SC_global->simu_mon.c_probe));
            (*(SC_global->simu_mon.pp_probe))   = (*(SC_global->simu_mon.c_probe));
            SC_global->simu_mon.c_probe         = NULL;
            delete csa_p_net;
        }
		ai++;
    }
    //
    // do not add code bellow this line 
    if ( SC_global->simu_state.done ) { // free the previous results
        free( SC_global->simu_res.res );
    }
    SC_global->simu_res.n_algo      = SC_global->simu_mon.n_algo;
    SC_global->simu_res.n_point     = SC_global->simu_param.npoint;
    SC_global->simu_res.res         = res;
    SC_global->simu_state.done      = TRUE;
// Simu_canceled processing statement
Simu_canceled:
    SC_global->simu_state.running   = FALSE;
    free( SC_global->simu_mon.pp_probe );
    SC_global->simu_mon.pp_probe    = NULL;
    return 0;
}


int CSimu_monitor_dlg::DoModal() 
{
	// TODO: Add your specialized code here and/or call the base class

    // calls the base class method
    return CDialog::DoModal();
}


void CSimu_monitor_dlg::OnCancel() 
{
	// TODO: Add extra cleanup here
    SC->simu_state.canceled = TRUE;
    KillTimer( m_timer_id );
	CDialog::OnCancel();
}


void CSimu_monitor_dlg::OnTimer(UINT nIDEvent) 
{
    // TODO: Add your message handler code here and/or call default
    if ( nIDEvent != MONITOR_TIMER_ID ) return;
    // calls base class implementation first 
    CDialog::OnTimer(nIDEvent);
    if ( SC->simu_state.running && SC->simu_mon.pp_probe && SC->simu_mon.c_point > 0 ) {
        // updates all the simu_mon observatioin field 
        m_algo  = CString(SC->simu_mon.c_algo_name);
        m_ant   = SC->simu_mon.pp_probe->ant;
	    m_apl   = SC->simu_mon.pp_probe->apl;
	    m_asd   = SC->simu_mon.pp_probe->asd;
	    m_nes   = SC->simu_mon.pp_probe->nes;
	    m_npb   = SC->simu_mon.pp_probe->npb;
	    m_npg   = SC->simu_mon.pp_probe->npg;
	    m_npl   = SC->simu_mon.pp_probe->npl;
	    m_npt   = SC->simu_mon.pp_probe->npt;
	    m_nra   = SC->simu_mon.pp_probe->nra;
	    m_nsg   = SC->simu_mon.pp_probe->nsg;
	    m_nsr   = SC->simu_mon.pp_probe->nsr;
	    m_q     = SC->simu_mon.pp_probe->q;
	    m_ro    = SC->simu_mon.pp_probe->ro;
	    m_tsd   = SC->simu_mon.pp_probe->tsd;
	    m_point = SC->simu_mon.c_point;
    }
    else {  
        // just updates the algorithm name
        m_algo  = CString(SC->simu_mon.c_algo_name);
    }
    // updates the simulation progress pos if possible 
    if (    SC->simu_mon.n_algo     && SC->simu_param.npoint && 
            SC->simu_param.nslot    && SC->simu_mon.c_probe ) 
    {
        double algo_step    = 100.0 / SC->simu_mon.n_algo;
        double point_step   = algo_step / SC->simu_param.npoint;
        double algo_base    = ( SC->simu_mon.c_algo - 1 ) * algo_step;
        double point_base   = algo_base + SC->simu_mon.c_point * point_step;

        m_simu_progress.SetPos( (UINT)( point_base + 
            point_step * SC->simu_mon.c_probe->nsg / SC->simu_param.nslot ) );
    }
    UpdateData( FALSE );
    //checks if the simulation is done
    if ( !SC->simu_state.running && SC->simu_state.done ) {
        SC->simu_state.canceled = FALSE;
        KillTimer( m_timer_id );
        CDialog::OnOK();
    }
}

BOOL CSimu_monitor_dlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
    SECURITY_ATTRIBUTES sa  = { sizeof(SECURITY_ATTRIBUTES), NULL, TRUE };
    SC->simu_state.canceled = FALSE;
    m_simu_th   = CreateThread( &sa, 0, simulate, SC, 0, &m_simu_tid );
    m_timer_id  = SetTimer( MONITOR_TIMER_ID, MONITOR_TIMER_DELAY, NULL );
    assert( m_timer_id );
    m_simu_progress.SetRange( 0, 100 );
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}
