/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      wsasimDoc.cpp : implementation of the CWsasimDoc class
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/



// wsasimDoc.cpp : implementation of the CWsasimDoc class
//

#include "stdafx.h"
#include "wsasim.h"

#include "MainFrm.h"
#include "wsasimDoc.h"
#include "wsasimView.h"
#include "Simu_monitor_dlg.h"
#include "Simu_param_dlg.h"
#include "Simu_log_dlg.h"
#include "assert.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWsasimDoc

IMPLEMENT_DYNCREATE(CWsasimDoc, CDocument)

BEGIN_MESSAGE_MAP(CWsasimDoc, CDocument)
	//{{AFX_MSG_MAP(CWsasimDoc)
	ON_COMMAND(ID_SIMU_EDIT, OnSimuEdit)
	ON_UPDATE_COMMAND_UI(ID_SIMU_EDIT, OnUpdateSimuEdit)
	ON_COMMAND(ID_SIMU_RUN, OnSimuRun)
	ON_UPDATE_COMMAND_UI(ID_SIMU_RUN, OnUpdateSimuRun)
	ON_COMMAND(ID_SIMU_LOG, OnSimuLog)
	ON_UPDATE_COMMAND_UI(ID_SIMU_LOG, OnUpdateSimuLog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWsasimDoc construction/destruction

CWsasimDoc::CWsasimDoc()
{
    // make sure the simulation result pointer is null at the begining
    SC.simu_res.res         = NULL;
}


CWsasimDoc::~CWsasimDoc()
{
}

BOOL CWsasimDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
    // default value of the simulation context */
    SC.net_param.n_station  = 10;
    SC.net_param.n_size     = 10;
    SC.simu_param.filled    = FALSE;
    SC.simu_param.rsa       = TRUE;
    SC.simu_param.csa       = FALSE;
    SC.simu_param.csa_p     = FALSE;
    SC.simu_param.ant       = TRUE;
    SC.simu_param.apl       = FALSE;
    SC.simu_param.asd       = FALSE;
    SC.simu_param.q         = FALSE;
    SC.simu_param.npoint    = 10;
    SC.simu_param.nslot     = 10000;
    SC.simu_mon.n_algo      = 0;
    SC.simu_mon.c_algo      = 0;
    SC.simu_mon.c_point     = 0;
    SC.simu_mon.c_algo_name = " ";
    SC.simu_mon.c_probe     = NULL;
    SC.simu_mon.pp_probe    = NULL;
    SC.simu_res.n_algo      = 0;
    SC.simu_res.n_point     = 0;
    if ( SC.simu_res.res ) {
        free( SC.simu_res.res );
        SC.simu_res.res = NULL;
    }
    SC.simu_state.running   = FALSE;
    SC.simu_state.canceled  = FALSE;
    SC.simu_state.done      = FALSE;
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CWsasimDoc serialization

void CWsasimDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())	// Stores the simulation Data
	{
		// Saves the network generic parameters
		ar.Write( &(SC.net_param), sizeof( WDM_Net_Param ) );
		// Saves the Simulation Param
		ar.Write( &(SC.simu_param), sizeof( WDM_Simu_Param ) );
		// Saves the Simulation Results
		ar.Write( &(SC.simu_res.n_algo), sizeof( int ) );
		ar.Write( &(SC.simu_res.n_point), sizeof( int ) );
		ar.Write(	SC.simu_res.res, 
					SC.simu_res.n_algo * SC.simu_res.n_point * sizeof( WDM_Probe ) );
		// Saves the Simulation State
		ar.Write( &(SC.simu_state), sizeof( WDM_Simu_State ) );
	}
	else
	{					// Reads the Simulation Data
		// Reads the network generic parameters
		ar.Read( &(SC.net_param), sizeof( WDM_Net_Param ) );
		// Reads the Simulation Param
		ar.Read( &(SC.simu_param), sizeof( WDM_Simu_Param ) );
		// Reads the Simulation Results
		ar.Read( &(SC.simu_res.n_algo), sizeof( int ) );
		ar.Read( &(SC.simu_res.n_point), sizeof( int ) );
		if ( SC.simu_res.res ) free( SC.simu_res.res );
		SC.simu_res.res = ( WDM_Probe * )malloc( 
				SC.simu_res.n_algo * SC.simu_res.n_point * sizeof( WDM_Probe ) );
		assert( SC.simu_res.res );
		ar.Read(	SC.simu_res.res, 
					SC.simu_res.n_algo * SC.simu_res.n_point * sizeof( WDM_Probe ) );
		// Reads the Simulation State
		ar.Read( &(SC.simu_state), sizeof( WDM_Simu_State ) );
	}
}

/////////////////////////////////////////////////////////////////////////////
// CWsasimDoc diagnostics

#ifdef _DEBUG
void CWsasimDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CWsasimDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWsasimDoc commands

void CWsasimDoc::OnSimuEdit() 
{
	// TODO: Add your command handler code here
    // shows the param dialog box
    CMainFrame  *pFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
    // Creates new param dialog
    CSimu_param_dlg *param_dlg  = new CSimu_param_dlg( pFrame, &SC );
    if ( param_dlg->DoModal( ) == IDOK ) SetModifiedFlag(); 
    delete param_dlg;
}

void CWsasimDoc::OnUpdateSimuEdit(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( !SC.simu_state.running ) pCmdUI->Enable( TRUE );
    else pCmdUI->Enable( FALSE );

}

void CWsasimDoc::OnSimuRun() 
{
    // TODO: Add your command handler code here
    CMainFrame  *pFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
    // Creates new simulation dialog
	UpdateAllViews( NULL); 
    CSimu_monitor_dlg *monitor_dlg  = new CSimu_monitor_dlg( pFrame, &SC );
    if ( monitor_dlg->DoModal() == IDOK ) UpdateAllViews( NULL); 
    delete monitor_dlg;
	
}

void CWsasimDoc::OnUpdateSimuRun(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( SC.simu_param.filled && !SC.simu_state.running ) pCmdUI->Enable( TRUE );
    else pCmdUI->Enable( FALSE );	
}

void CWsasimDoc::OnSimuLog() 
{
	// TODO: Add your command handler code here
    // shows the log dialog box
    CMainFrame  *pFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
    // Creates new log dialog
    CSimu_log_dlg *log_dlg  = new CSimu_log_dlg( pFrame, &SC );
    log_dlg->DoModal( );
    delete log_dlg;
}

void CWsasimDoc::OnUpdateSimuLog(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( SC.simu_state.done && !SC.simu_state.running ) pCmdUI->Enable( TRUE );
    else pCmdUI->Enable( FALSE );	
}
