// wsasimDoc.h : interface of the CWsasimDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_WSASIMDOC_H__5AC787EF_D180_11D1_85F9_B9B8623EBD1C__INCLUDED_)
#define AFX_WSASIMDOC_H__5AC787EF_D180_11D1_85F9_B9B8623EBD1C__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "..\sasim\wdm_network.h"


/*--------------------------------------------------------------------
 |  Type definitions bellow this line
 +--------------------------------------------------------------------*/
    
    /*** Network Generic Parameters ***/
typedef struct {
	UINT	    n_station;  /* number of station in the network */
    UINT	    n_size;     /* size of the network in slot ( k ) */             
} WDM_Net_Param;


    /*** Simulation Parameters ***/
typedef struct {
    // indicates if the simualtion parameters have been filled
    BOOL    filled;
    // type of Algorithm being simulated     
	BOOL	rsa;    /* indicates if rsa algorithm is selected */
    BOOL	csa;    /* indicates if csa algorithm is selected */
	BOOL	csa_p;  /* indicates if csa+ algorithm is selected */
    // curves selected 
    BOOL	ant;    /* indicates if the average network throughput is to be drawn */
	BOOL	apl;    /* indicates if the average packet loss is to be drawn */
	BOOL	asd;    /* indicates if the average slot delay is to be drawn */
	BOOL	q;      /* indicates if the reservation success rate is to be drawn */
    // points to simulate
    UINT	npoint; /* number of point to simulate */    
	UINT	nslot;  /* number of slot to genreate per point */
} WDM_Simu_Param;


    /*** Simulation Monitor ***/
typedef struct {
    int         n_algo;         /* indicates the number of algorithm selected */
    int         c_algo;         /* indicates the pos of the ( current ) algorithm being simulated */
    int         c_point;        /* current point being simulated */
    char        *c_algo_name;   /* name of the ( current ) algorithm being simulated */
    WDM_Probe   *c_probe;       /* Current Probe */    
    WDM_Probe   *pp_probe;      /* Previous Point Simulation end Probe */    
} WDM_Simu_Mon;


    /*** Simulation Results ***/
typedef struct {
    int         n_algo;     /* indicates the number of algorithm selected */
    int         n_point;    /* number of point in the simulation */
    WDM_Probe   *res;       /* Result Probes */    
} WDM_Simu_Res;


    /*** Simulation State ***/
typedef struct {
    BOOL    running;        /* indicates if the simulation is running */
    BOOL    canceled;       /* indicates if the simulation has been canceled */
    BOOL    done;           /* indicates if the simulation has been done */
} WDM_Simu_State;


    /*** Simulation Context ***/
typedef struct {
    WDM_Net_Param   net_param;  /* Network Generic Parameters */
    WDM_Simu_Param  simu_param; /* Simulation Parameter */
    WDM_Simu_Mon    simu_mon;   /* Simulation Monitor */    
    WDM_Simu_State  simu_state; /* indicates the simulation state */
    WDM_Simu_Res    simu_res;   /* Result Probes of Simulation */
} WDM_Simu_Context;



class CWsasimDoc : public CDocument
{
protected: // create from serialization only
	CWsasimDoc();
	DECLARE_DYNCREATE(CWsasimDoc)

// Attributes
public:
	WDM_Simu_Context SC;    // Simulation Context

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CWsasimDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CWsasimDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CWsasimDoc)
	afx_msg void OnSimuEdit();
	afx_msg void OnUpdateSimuEdit(CCmdUI* pCmdUI);
	afx_msg void OnSimuRun();
	afx_msg void OnUpdateSimuRun(CCmdUI* pCmdUI);
	afx_msg void OnSimuLog();
	afx_msg void OnUpdateSimuLog(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_WSASIMDOC_H__5AC787EF_D180_11D1_85F9_B9B8623EBD1C__INCLUDED_)
