/*--------------------------------------------------------------------
 |		Copyright 1998 ( ralao@venus.org ) 
 +--------------------------------------------------------------------
 |
 |      wsasimView.cpp : implementation of the CWsasimView class
 |      by ralao ( ENST 1998 )
 |
 |		This file may be copied, modified and freely distributed as
 |		long as this notice is provided.
 |
 |		Disclaimer :
 |      ------------
 |			This file is part of a Network simulator program. It is 
 |		provided AS IS. Though it has been tested and found harmless 
 |		for any computer, venus.org may not be held liable for any 
 |		damage resulting in the usage of this file.
 |
 +---------------------------------------------------------------------
 |		For assistance, you may contact ralao@venus.org
 +--------------------------------------------------------------------*/



// wsasimView.cpp : implementation of the CWsasimView class
//

#include "stdafx.h"
#include "wsasim.h"

#include "wsasimDoc.h"
#include "wsasimView.h"
#include "limits.h"
#include "assert.h"


// key extended ascii code def 
#define UP_KEY			72
#define DOWN_KEY		80
#define LEFT_KEY		75
#define RIGHT_KEY		77

#define ARROW_SCROLL	100


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWsasimView

IMPLEMENT_DYNCREATE(CWsasimView, CScrollView)

BEGIN_MESSAGE_MAP(CWsasimView, CScrollView)
	//{{AFX_MSG_MAP(CWsasimView)
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWsasimView construction/destruction

CWsasimView::CWsasimView()
{
}

CWsasimView::~CWsasimView()
{
}

BOOL CWsasimView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CWsasimView drawing

//
// Just draws the Logo of the Wsasim
//
void CWsasimView::DrawLogo( CDC* pDC )
{
    // Creates a new big font and sets it as curent font
    CFont   BigFont;
    BigFont.CreatePointFont( 310, CString( "Arial" ), pDC );
    CFont   *OldFont    = ( CFont * ) pDC->SelectObject( &BigFont );


    pDC->DrawText(  CString( " Windows Scheduling Algorithm " ) , 
                    CRect( 0, -300, 2100, -500 ), DT_CENTER );
    
    pDC->DrawText(  CString( " Simulator " ) , 
                    CRect( 0, -500, 2100, -700 ), DT_CENTER );
    
    // restores the old font
    pDC->SelectObject( OldFont );
    pDC->DrawText(  CString( " Copyright Rachad ALAO ( ralao@venus.org ) 1998" ) , 
                    CRect( 0, -700, 2100, -900 ), DT_CENTER );
}

//
// draws Average Network Throughput 
//
void CWsasimView::draw_ant( int x, int y, CDC* pDC, WDM_Simu_Context *SC )
{
    int i = 0;
    // Select Large Black Pen
    CPen    *OldPen = ( CPen * ) pDC->SelectObject( &Black_L );
    // Draws the title of the result frame 
    pDC->DrawText(  CString( " Average Network Throughput " ) , 
                    CRect( x + 100, y - 30, x + 800, y - 100 ), DT_CENTER );
    // Draws the axes and the graph
    int		xo	= x + 50;
    int		yo  = y - 1000;
    double	dx  = 850.0;
    double	dy  = 850.0;
		// draws vertical axe
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo, yo + 850 );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo, yo + i * 85 );
		pDC->LineTo( xo + 850, yo + i * 85 );
    }
		// draws horizontal axe
    pDC->SelectObject( &Black_L );
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo + 850, yo );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo + i * 85, yo );
		pDC->LineTo( xo + i * 85, yo + 850 );
    }
		// writes axes' scales
    pDC->DrawText( CString( "0" ), CRect( xo - 50, yo - 10, xo - 10, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "0.1" ), CRect( xo + 55, yo - 10, xo + 115, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "1" ), CRect( xo + 820, yo - 10, xo + 880, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "ro" ), CRect( xo + 800, yo + 60, xo + 900, yo + 20 ), DT_CENTER );
    pDC->DrawText( CString( "0.1" ), CRect( xo - 60, yo + 125, xo - 10, yo + 55 ), DT_CENTER );
    pDC->DrawText( CString( "1" ), CRect( xo - 50, yo + 880, xo - 10, yo + 820 ), DT_CENTER );
    pDC->DrawText( CString( "S" ), CRect( xo + 20, yo + 880, xo + 60, yo + 820 ), DT_CENTER );
    int ai  = 0;    // algo_index
		// draws the curves
    if ( SC->simu_param.rsa ) {         // RSA has been simulated, draws the results
        pDC->SelectObject( &Red );      // Red pen is for RSA curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].ant * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa ) {         // CSA has been simulated, draws the results
        pDC->SelectObject( &Green );    // Green pen is for CSA curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].ant * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa_p ) {       // CSA+ has been simulated, draws the results
        pDC->SelectObject( &Blue );     // Blue pen is for CSA+ curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].ant * dy ) );
        }
        ai++;
    }
    // Restores OldPen
    pDC->SelectObject( OldPen );
}


//
// draws Average Slot Delay 
//
void CWsasimView::draw_asd( int x, int y, CDC* pDC, WDM_Simu_Context *SC )
{
    int i = 0;
    // Select Large Black Pen
    CPen    *OldPen = ( CPen * ) pDC->SelectObject( &Black_L );
    // Draws the title of the result frame 
    pDC->DrawText(  CString( " Average Slot Delay " ) , 
                    CRect( x + 100, y - 30, x + 800, y - 100 ), DT_CENTER );
    // Draws the axes and the graph
    int xo  = x + 50;
    int yo  = y - 1000;
		// Autoscales the graph
	int max_asd	= INT_MIN;
	for ( i =0; i < SC->simu_res.n_point * SC->simu_res.n_algo; i++ ) {
		if ( SC->simu_res.res[i].asd > max_asd ) {
			max_asd	= (int)(SC->simu_res.res[i].asd);
		}
	}
    double	dx  = 850.0;
    double	dy  = ( max_asd == 0 ) ? 850.0 : 850.0 / max_asd;
		// draws vertical axe
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo, yo + 850 );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo, yo + i * 85 );
		pDC->LineTo( xo + 850, yo + i * 85 );
    }
		// draws horizontal axe
    pDC->SelectObject( &Black_L );
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo + 850, yo );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo + i * 85, yo );
		pDC->LineTo( xo + i * 85, yo + 850 );
    }
    // writes axes' scales
    pDC->DrawText( CString( "0" ), CRect( xo - 50, yo - 10, xo - 10, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "0.1" ), CRect( xo + 55, yo - 10, xo + 115, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "1" ), CRect( xo + 820, yo - 10, xo + 880, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "ro" ), CRect( xo + 800, yo + 60, xo + 900, yo + 20 ), DT_CENTER );
    char str[40];
	sprintf( str, "%.1f", max_asd / 10.0 );
	pDC->DrawText( CString( str ), CRect( xo - 100, yo + 125, xo - 10, yo + 55 ), DT_CENTER );
    sprintf( str, "%d", max_asd );
	pDC->DrawText( CString( str ), CRect( xo - 100, yo + 880, xo - 10, yo + 820 ), DT_CENTER );
    pDC->DrawText( CString( "asd" ), CRect( xo + 20, yo + 880, xo + 100, yo + 820 ), DT_CENTER );
    int ai  = 0;    // algo_index
    // draws the curves
    if ( SC->simu_param.rsa ) {         // RSA has been simulated, draws the results
        pDC->SelectObject( &Red );      // Red pen is for RSA curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].asd * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa ) {         // CSA has been simulated, draws the results
        pDC->SelectObject( &Green );    // Green pen is for CSA curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].asd * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa_p ) {       // CSA+ has been simulated, draws the results
        pDC->SelectObject( &Blue );     // Blue pen is for CSA+ curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].asd * dy ) );
        }
        ai++;
    }
    // Restores OldPen
    pDC->SelectObject( OldPen );
}


//
// draws Average Packet Loss
//
void CWsasimView::draw_apl( int x, int y, CDC* pDC, WDM_Simu_Context *SC )
{
    int i = 0;
    // Select Large Black Pen
    CPen    *OldPen = ( CPen * ) pDC->SelectObject( &Black_L );
    // Draws the title of the result frame 
    pDC->DrawText(  CString( " Average Packet Loss " ) , 
                    CRect( x + 100, y - 30, x + 800, y - 100 ), DT_CENTER );
    // Draws the axes and the graph
    int		xo	= x + 50;
    int		yo  = y - 1000;
    double	dx  = 850.0;
    double	dy  = 850.0;
		// draws vertical axe
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo, yo + 850 );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo, yo + i * 85 );
		pDC->LineTo( xo + 850, yo + i * 85 );
    }
		// draws horizontal axe
    pDC->SelectObject( &Black_L );
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo + 850, yo );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo + i * 85, yo );
		pDC->LineTo( xo + i * 85, yo + 850 );
    }
		// writes axes' scales
    pDC->DrawText( CString( "0" ), CRect( xo - 50, yo - 10, xo - 10, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "0.1" ), CRect( xo + 55, yo - 10, xo + 115, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "1" ), CRect( xo + 820, yo - 10, xo + 880, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "ro" ), CRect( xo + 800, yo + 60, xo + 900, yo + 20 ), DT_CENTER );
    pDC->DrawText( CString( "0.1" ), CRect( xo - 60, yo + 125, xo - 10, yo + 55 ), DT_CENTER );
    pDC->DrawText( CString( "1" ), CRect( xo - 50, yo + 880, xo - 10, yo + 820 ), DT_CENTER );
    pDC->DrawText( CString( "apl" ), CRect( xo + 20, yo + 880, xo + 100, yo + 820 ), DT_CENTER );
    int ai  = 0;    // algo_index
		// draws the curves
    if ( SC->simu_param.rsa ) {         // RSA has been simulated, draws the results
        pDC->SelectObject( &Red );      // Red pen is for RSA curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].apl * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa ) {         // CSA has been simulated, draws the results
        pDC->SelectObject( &Green );    // Green pen is for CSA curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].apl * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa_p ) {       // CSA+ has been simulated, draws the results
        pDC->SelectObject( &Blue );     // Blue pen is for CSA+ curves    
        pDC->MoveTo( xo, yo );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].apl * dy ) );
        }
        ai++;
    }
    // Restores OldPen
    pDC->SelectObject( OldPen );
}



//
// draws Reservation Success Rate
//
void CWsasimView::draw_q( int x, int y, CDC* pDC, WDM_Simu_Context *SC )
{
    int i = 0;
    // Select Large Black Pen
    CPen    *OldPen = ( CPen * ) pDC->SelectObject( &Black_L );
    // Draws the title of the result frame 
    pDC->DrawText(  CString( " Reservation Success Rate ( q ) " ) , 
                    CRect( x + 100, y - 30, x + 800, y - 100 ), DT_CENTER );
    // Draws the axes and the graph
    int		xo	= x + 50;
    int		yo  = y - 1000;
    double	dx  = 850.0;
    double	dy  = 850.0;
		// draws vertical axe
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo, yo + 850 );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo, yo + i * 85 );
		pDC->LineTo( xo + 850, yo + i * 85 );
    }
		// draws horizontal axe
    pDC->SelectObject( &Black_L );
    pDC->MoveTo( xo, yo );
    pDC->LineTo( xo + 850, yo );
    pDC->SelectObject( &Gray );
    for ( i = 1; i <= 10; i ++ ) {
        pDC->MoveTo( xo + i * 85, yo );
		pDC->LineTo( xo + i * 85, yo + 850 );
    }
		// writes axes' scales
    pDC->DrawText( CString( "0" ), CRect( xo - 50, yo - 10, xo - 10, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "0.1" ), CRect( xo + 55, yo - 10, xo + 115, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "1" ), CRect( xo + 820, yo - 10, xo + 880, yo - 100 ), DT_CENTER );
    pDC->DrawText( CString( "ro" ), CRect( xo + 800, yo + 60, xo + 900, yo + 20 ), DT_CENTER );
    pDC->DrawText( CString( "0.1" ), CRect( xo - 60, yo + 125, xo - 10, yo + 55 ), DT_CENTER );
    pDC->DrawText( CString( "1" ), CRect( xo - 50, yo + 880, xo - 10, yo + 820 ), DT_CENTER );
    pDC->DrawText( CString( "q" ), CRect( xo + 20, yo + 880, xo + 100, yo + 820 ), DT_CENTER );
    int ai  = 0;    // algo_index
		// draws the curves
    if ( SC->simu_param.rsa ) {         // RSA has been simulated, draws the results
        pDC->SelectObject( &Red );      // Red pen is for RSA curves    
        pDC->MoveTo( xo, yo + 850 );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].q * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa ) {         // CSA has been simulated, draws the results
        pDC->SelectObject( &Green );    // Green pen is for CSA curves    
        pDC->MoveTo( xo, yo + 850 );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].q * dy ) );
        }
        ai++;
    }
    if ( SC->simu_param.csa_p ) {       // CSA+ has been simulated, draws the results
        pDC->SelectObject( &Blue );     // Blue pen is for CSA+ curves    
        pDC->MoveTo( xo, yo + 850 );
        int np  = SC->simu_res.n_point;
        for ( i = 0; i < np; i++ ) {
            pDC->LineTo(    (int)( xo + SC->simu_res.res[ai*np + i].ro * dx ), 
                            (int)( yo + SC->simu_res.res[ai*np + i].q * dy ) );
        }
        ai++;
    }
    // Restores OldPen
    pDC->SelectObject( OldPen );
}


//
// draws Reservation Success Rate
//
void CWsasimView::draw_legend( int x, int y, CDC* pDC, WDM_Simu_Context *SC )
{
	char tmp[80];	// a temporary line of text
    
	// Select Large Black Pen
    CPen    *OldPen = ( CPen * ) pDC->SelectObject( &Black );
	// y += 50;
	if ( SC->simu_param.rsa ) {         // RSA has been simulated, draws the results
		pDC->SelectObject( &Red );		// Red pen is for RSA curves    
		pDC->MoveTo( 100, y - 25 );
		pDC->LineTo( 200, y - 25 );
		pDC->DrawText(  CString( "RSA" ) , CRect( 250, y , 500, y - 50 ), DT_LEFT );
	}
	if ( SC->simu_param.csa ) {         // CSA has been simulated, draws the results
		pDC->SelectObject( &Green );    // Green pen is for CSA curves    
		pDC->MoveTo( 100, y - 75 );
		pDC->LineTo( 200, y - 75 );
		pDC->DrawText(  CString( "CSA" ) , CRect( 250, y - 50 , 500, y - 100 ), DT_LEFT );
	}
	if ( SC->simu_param.csa_p ) {       // CSA+ has been simulated, draws the results
		pDC->SelectObject( &Blue );		//  Blue pen is for CSA+ curves    
		pDC->MoveTo( 100, y - 125 );
		pDC->LineTo( 200, y - 125 );
		pDC->DrawText(  CString( "CSA+" ) , CRect( 250, y - 100 , 500, y - 150 ), DT_LEFT );
	}
	sprintf( tmp, "Number of Station in Network ( N ) : %d", SC->net_param.n_station );
	pDC->DrawText(  CString( tmp ) , CRect( 500, y , 1500, y - 50 ), DT_LEFT );
	sprintf( tmp, "Network Size ( k ) : %d", SC->net_param.n_size );
	pDC->DrawText(  CString( tmp ) , CRect( 500, y - 50, 1500, y - 100 ), DT_LEFT );
	sprintf( tmp, "Number of Simulation Point : %d", SC->simu_param.npoint );
	pDC->DrawText(  CString( tmp ) , CRect( 500, y - 100, 1500, y - 150 ), DT_LEFT );
	sprintf( tmp, "Number of Slot Simulated Per Point : %d", SC->simu_param.nslot );
	pDC->DrawText(  CString( tmp ) , CRect( 500, y - 150, 1500, y - 200 ), DT_LEFT );

	pDC->SelectObject( OldPen );

}


//
//	calc_graph_pos : just calculates the pos of the next graph
//
static void calc_ngp( int &x, int &y )
{
	if ( x == 1100 ) {
		x =		100;
		y +=	-1100;
	}
	else {
		x =		1100;
		y +=	0;
	}
}


//
// OnDraw : WM_PAINT message handler
//
void CWsasimView::OnDraw(CDC* pDC)
{
	CWsasimDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
    // Creates a new big font and sets it as curent font
    CFont   BigFont;
    BigFont.CreatePointFont( 100, CString( "Arial" ), pDC );
    CFont   *OldFont    = ( CFont * ) pDC->SelectObject( &BigFont );

	if ( pDoc->SC.simu_state.done == FALSE ) {
		// Draws the WSASIM logo
        DrawLogo( pDC ); 
    }
    else {
        // Draws the WSASIM result title
        // Select Large Black Pen
        CPen    *OldPen = ( CPen * ) pDC->SelectObject( &Black_L );
        pDC->Rectangle( CRect( 850, -100, 1250, -200 ) );
        pDC->DrawText(  CString( " WSASIM Results " ) , 
                        CRect( 850, -130, 1250, -200 ), DT_CENTER );
        pDC->SelectObject( OldPen );
        // Draws the Simulation results
		int x = 100;
		int y = -250;
        if ( pDoc->SC.simu_param.ant ) {
			draw_ant( x, y, pDC, &(pDoc->SC) );
			calc_ngp( x, y );
		}
		if ( pDoc->SC.simu_param.asd ) {
			draw_asd( x, y, pDC, &(pDoc->SC) );
			calc_ngp( x, y );
		}
		if ( pDoc->SC.simu_param.apl ) {
			draw_apl( x, y, pDC, &(pDoc->SC) );
			calc_ngp( x, y );
		}
		if ( pDoc->SC.simu_param.q ) {
			draw_q( x, y, pDC, &(pDoc->SC) );
			calc_ngp( x, y );
		}
		// draws the legend of the graphs
		if ( y == -250 ) calc_ngp( x, y ); // hack for draw_legend !!!
		draw_legend( x, y, pDC, &(pDoc->SC) );
    }
	pDC->SelectObject( OldFont );
}

/////////////////////////////////////////////////////////////////////////////
// CWsasimView printing

BOOL CWsasimView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CWsasimView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CWsasimView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CWsasimView diagnostics

#ifdef _DEBUG
void CWsasimView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CWsasimView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CWsasimDoc* CWsasimView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CWsasimDoc)));
	return (CWsasimDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWsasimView message handlers

void CWsasimView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	
	// TODO: Add your specialized code here and/or call the base class
    SetScrollSizes( MM_LOMETRIC, CSize( 2100, 2970 ) );	
    // Inits the pens
    Black.DeleteObject();
    Black.CreatePen( PS_SOLID, 1, RGB( 0, 0, 0) );
    Black_L.DeleteObject();
    Black_L.CreatePen( PS_SOLID, 10, RGB( 0, 0, 0) );
	Gray.DeleteObject();
    Gray.CreatePen( PS_DOT, 1, RGB( 200, 200, 200) );
    Red.DeleteObject();
    Red.CreatePen( PS_SOLID, 1, RGB( 255, 0, 0) );
    Green.DeleteObject();
    Green.CreatePen( PS_DASH, 1, RGB( 0, 255, 0) );
    Blue.DeleteObject();
    Blue.CreatePen( PS_DASHDOT, 1, RGB( 0, 0, 255) );
}



void CWsasimView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	char	up_code		= ( char )( ( nChar & 0x0000FF00 ) >> 8 );	// up mask
	char	down_code	= ( char )( nChar & 0x000000FF );			// down mask

	CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
}
